package com.lightinit.hsdataportal.common;

import com.alibaba.druid.support.json.JSONUtils;
import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.lightinit.hsdataportal.common.HttpUtil;
import com.lightinit.hsdataportal.handler.GlobalExceptionHandler;
import com.sun.org.apache.bcel.internal.generic.NEW;
import net.sf.json.JSON;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * @Author: sunxiang
 * @Date: 2019/2/28 17:07
 * @Description:
 **/
public class AuthenticUtil {

    private static final Logger logger = LoggerFactory.getLogger(AuthenticUtil.class);
    public static final String DEF_CHATSET = "UTF-8";
    public static final int DEF_CONN_TIMEOUT = 30000;
    public static final int DEF_READ_TIMEOUT = 30000;
    public static String userAgent =  "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36";


    public static final String METHOD="GET";
    public static  String IDCARD_STRURL="";
    public static  String IDCARD_APPKEY ="";
    public static  String BANKCARD_STRURL="";
    public static  String BANKCARD_APPKEY ="";
    public static  String PHONE_STRURL="";
    public static  String PHONE_APPKEY ="";
    static {
        InputStream inputStream=null;
        try {
            inputStream= AuthenticUtil.class.getClassLoader().getResourceAsStream("authentic.properties");
            Properties properties = new Properties();
            properties.load(inputStream);
            IDCARD_STRURL = properties.getProperty("authentic.idcard.strUrl");
            IDCARD_APPKEY = properties.getProperty("authentic.idcard.appkey");
            BANKCARD_STRURL = properties.getProperty("authentic.bank.strUrl");
            BANKCARD_APPKEY = properties.getProperty("authentic.bank.appkey");
            PHONE_STRURL = properties.getProperty("authentic.phone.strUrl");
            PHONE_APPKEY = properties.getProperty("authentic.phone.appkey");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if(inputStream!=null){
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    public  static boolean authenticByIdCard(String username,String idcard){
        HashMap<String, String> params = new HashMap<>();
        params.put("key",IDCARD_APPKEY);
        params.put("idcard",idcard);
        params.put("realname",username);
        try {
            String result = net(IDCARD_STRURL,params, METHOD);
            JSONObject object = JSONObject.fromObject(result);
            if(object.getInt("error_code")==0) {
                Object obj = object.get("result");
                System.out.println(obj);
                JSONObject jsonObject = JSONObject.fromObject(obj);
                if (jsonObject.getInt("res") == 1) {
                    return true;
                }
            }else{
                logger.error("调用身份验证网站错误，请查看对应码解释：错误码是："+object.getInt("error_code"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }


    public static boolean authenticByBankCard(String username,String bankCard){
        Map params = new HashMap();//请求参数
        params.put("bankcard",bankCard);//银行卡卡号
        params.put("realname",username);//姓名(需utf8编码的urlencode)
        params.put("key",BANKCARD_APPKEY);//应用APPKEY(应用详细页查询)
        try {
            String result =net(BANKCARD_STRURL, params, METHOD);
            JSONObject object = JSONObject.fromObject(result);
            if(object.getInt("error_code")==0) {
                Object obj = object.get("result");
                System.out.println(obj);
                JSONObject jsonObject = JSONObject.fromObject(obj);
                if (jsonObject.getInt("res") == 1) {
                    return true;
                }
            }else{
                logger.error("调用身份验证网站错误，请查看对应码解释：错误码是："+object.getInt("error_code"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }


    public static boolean authenticByPhone(String username,String idCard,String phone){
            System.out.println(PHONE_APPKEY);
            System.out.println(PHONE_STRURL);
            Map params = new HashMap();//请求参数
            params.put("idcard",idCard);//银行卡卡号
            params.put("realname",username);//姓名(需utf8编码的urlencode)
            params.put("mobile",phone);//姓名(需utf8编码的urlencode)
            params.put("key",PHONE_APPKEY);//应用APPKEY(应用详细页查询)
            try {
                String result =net(PHONE_STRURL, params, METHOD);
                JSONObject object = JSONObject.fromObject(result);
                if(object.getInt("error_code")==0) {
                    Object obj = object.get("result");
                    System.out.println(obj);
                    JSONObject jsonObject = JSONObject.fromObject(obj);
                    if (jsonObject.getInt("res") == 1) {
                        return true;
                    }
                }else{
                    logger.error("调用身份验证网站错误，请查看对应码解释：错误码是："+object.getInt("error_code"));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            return false;
        }




    /**
     *
     * @param strUrl 请求地址
     * @param params 请求参数
     * @param method 请求方法
     * @return  网络请求字符串
     * @throws Exception
     */
    public static String net(String strUrl, Map params,String method) throws Exception {
        HttpURLConnection conn = null;
        BufferedReader reader = null;
        String rs = null;
        try {
            StringBuffer sb = new StringBuffer();
            if(method==null || method.equals("GET")){
                strUrl = strUrl+"?"+urlencode(params);
            }
            URL url = new URL(strUrl);
            conn = (HttpURLConnection) url.openConnection();
            if(method==null || method.equals("GET")){
                conn.setRequestMethod("GET");
            }else{
                conn.setRequestMethod("POST");
                conn.setDoOutput(true);
            }
            conn.setRequestProperty("User-agent", userAgent);
            conn.setUseCaches(false);
            conn.setConnectTimeout(DEF_CONN_TIMEOUT);
            conn.setReadTimeout(DEF_READ_TIMEOUT);
            conn.setInstanceFollowRedirects(false);
            conn.connect();
            if (params!= null && method.equals("POST")) {
                try {
                    DataOutputStream out = new DataOutputStream(conn.getOutputStream());
                    out.writeBytes(urlencode(params));
                } catch (Exception e) {
                    // handle exception
                }
            }
            InputStream is = conn.getInputStream();
            reader = new BufferedReader(new InputStreamReader(is, DEF_CHATSET));
            String strRead = null;
            while ((strRead = reader.readLine()) != null) {
                sb.append(strRead);
            }
            rs = sb.toString();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (reader != null) {
                reader.close();
            }
            if (conn != null) {
                conn.disconnect();
            }
        }
        return rs;
    }

    //将map型转为请求参数型
    public static String urlencode(Map<String,Object>data) {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry i : data.entrySet()) {
            try {
                sb.append(i.getKey()).append("=").append(URLEncoder.encode(i.getValue()+"","UTF-8")).append("&");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
}
